@@ -17,7 +17,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written by Brian Fox (bfox@ai.mit.edu). */
 
@@ -36,5 +36,6 @@ REFERENCE **apropos_in_all_indices ();
 /* User visible functions declared in indices.c. */
 extern void info_index_search (), info_next_index_match ();
 extern void do_info_index_search ();
+extern int index_intry_exists ();
 
 #endif /* !_INDICES_H_ */
