@@ -1,7 +1,6 @@
-//                                  -*- C++ -*-
 /*
 
-Copyright (C) 1992, 1993, 1994, 1995 John W. Eaton
+Copyright (C) 1996 John W. Eaton
 
 This file is part of Octave.
 
@@ -17,42 +16,44 @@ for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
 #if !defined (octave_ComplexRowVector_h)
 #define octave_ComplexRowVector_h 1
 
-#include "Array.h"
+#if defined (__GNUG__)
+#pragma interface
+#endif
+
+#include "MArray.h"
 
 #include "mx-defs.h"
 
-extern "C++" {
-
-class ComplexRowVector : public Array<Complex>
+class ComplexRowVector : public MArray<Complex>
 {
 friend class ComplexColumnVector;
 
 public:
 
-  ComplexRowVector (void) : Array<Complex> () { }
-  ComplexRowVector (int n) : Array<Complex> (n) { }
-  ComplexRowVector (int n, const Complex& val) : Array<Complex> (n, val) { }
+  ComplexRowVector (void) : MArray<Complex> () { }
+  ComplexRowVector (int n) : MArray<Complex> (n) { }
+  ComplexRowVector (int n, const Complex& val) : MArray<Complex> (n, val) { }
   ComplexRowVector (const RowVector& a);
-  ComplexRowVector (const Array<Complex>& a) : Array<Complex> (a) { }
-  ComplexRowVector (const ComplexRowVector& a) : Array<Complex> (a) { }
+  ComplexRowVector (const MArray<Complex>& a) : MArray<Complex> (a) { }
+  ComplexRowVector (const ComplexRowVector& a) : MArray<Complex> (a) { }
 
   ComplexRowVector& operator = (const ComplexRowVector& a)
     {
-      Array<Complex>::operator = (a);
+      MArray<Complex>::operator = (a);
       return *this;
     }
 
-  int operator == (const ComplexRowVector& a) const;
-  int operator != (const ComplexRowVector& a) const;
+  bool operator == (const ComplexRowVector& a) const;
+  bool operator != (const ComplexRowVector& a) const;
 
-// destructive insert/delete/reorder operations
+  // destructive insert/delete/reorder operations
 
   ComplexRowVector& insert (const RowVector& a, int c);
   ComplexRowVector& insert (const ComplexRowVector& a, int c);
@@ -70,11 +71,11 @@ public:
 
   friend ComplexRowVector conj (const ComplexRowVector& a);
 
-// resize is the destructive equivalent for this one
+  // resize is the destructive equivalent for this one
 
   ComplexRowVector extract (int c1, int c2) const;
 
-// row vector by row vector -> row vector operations
+  // row vector by row vector -> row vector operations
 
   ComplexRowVector& operator += (const RowVector& a);
   ComplexRowVector& operator -= (const RowVector& a);
@@ -82,7 +83,7 @@ public:
   ComplexRowVector& operator += (const ComplexRowVector& a);
   ComplexRowVector& operator -= (const ComplexRowVector& a);
 
-// row vector by scalar -> row vector operations
+  // row vector by scalar -> row vector operations
 
   friend ComplexRowVector operator + (const ComplexRowVector& a, double s);
   friend ComplexRowVector operator - (const ComplexRowVector& a, double s);
@@ -94,7 +95,7 @@ public:
   friend ComplexRowVector operator * (const RowVector& a, const Complex& s);
   friend ComplexRowVector operator / (const RowVector& a, const Complex& s);
 
-// scalar by row vector -> row vector operations
+  // scalar by row vector -> row vector operations
 
   friend ComplexRowVector operator + (double s, const ComplexRowVector& a);
   friend ComplexRowVector operator - (double s, const ComplexRowVector& a);
@@ -106,7 +107,7 @@ public:
   friend ComplexRowVector operator * (const Complex& s, const RowVector& a);
   friend ComplexRowVector operator / (const Complex& s, const RowVector& a);
 
-// row vector by matrix -> row vector
+  // row vector by matrix -> row vector
 
   friend ComplexRowVector operator * (const ComplexRowVector& a,
 				      const ComplexMatrix& b);
@@ -114,7 +115,7 @@ public:
   friend ComplexRowVector operator * (const RowVector& a,
 				      const ComplexMatrix& b);
 
-// row vector by row vector -> row vector operations
+  // row vector by row vector -> row vector operations
 
   friend ComplexRowVector operator + (const ComplexRowVector& a,
 				      const RowVector& b);
@@ -136,33 +137,24 @@ public:
   friend ComplexRowVector quotient (const RowVector& a,
 				    const ComplexRowVector& b);
 
-// other operations
+  // other operations
 
-  friend ComplexRowVector map (c_c_Mapper f, const ComplexRowVector& a);
-  void map (c_c_Mapper f);
+  ComplexRowVector map (c_c_Mapper f) const;
+  RowVector map (d_c_Mapper f) const;
 
-  friend ComplexRowVector linspace (const Complex& x1, const Complex& x2,
-				    int n);
+  ComplexRowVector& apply (c_c_Mapper f);
 
   Complex min (void) const;
   Complex max (void) const;
 
-// i/o
+  // i/o
 
   friend ostream& operator << (ostream& os, const ComplexRowVector& a);
   friend istream& operator >> (istream& is, ComplexRowVector& a);
 
-#define KLUDGE_VECTORS
-#define TYPE Complex
-#define KL_VEC_TYPE ComplexRowVector
-#include "mx-kludge.h"
-#undef KLUDGE_VECTORS
-#undef TYPE
-#undef KL_VEC_TYPE
-
 private:
 
-  ComplexRowVector (Complex *d, int l) : Array<Complex> (d, l) { }
+  ComplexRowVector (Complex *d, int l) : MArray<Complex> (d, l) { }
 };
 
 // row vector by column vector -> scalar
@@ -171,13 +163,14 @@ Complex operator * (const ComplexRowVector& a, const ColumnVector& b);
 
 Complex operator * (const ComplexRowVector& a, const ComplexColumnVector& b);
 
-} // extern "C++"
+// other operations
+
+ComplexRowVector linspace (const Complex& x1, const Complex& x2, int n);
 
 #endif
 
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
-;;; page-delimiter: "^/\\*" ***
 ;;; End: ***
 */
