@@ -1,7 +1,7 @@
-// Matrix manipulations.                                 -*- C++ -*-
+// Matrix manipulations.
 /*
 
-Copyright (C) 1992, 1993, 1994, 1995 John W. Eaton
+Copyright (C) 1996 John W. Eaton
 
 This file is part of Octave.
 
@@ -17,16 +17,12 @@ for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
-/*
-
-Should probably say something here about why these classes are not
-represented by some sort of inheritance tree...
-
-*/
+// Should probably say something here about why these classes are not
+// represented by some sort of inheritance tree...
 
 #if !defined (octave_Matrix_h)
 #define octave_Matrix_h 1
@@ -40,6 +36,5 @@ represented by some sort of inheritance tree...
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
-;;; page-delimiter: "^/\\*" ***
 ;;; End: ***
 */
