@@ -42,7 +42,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 extern "C"
 {
-  double F77_FCN (dgamma, DGAMMA) (const double&);
+  int F77_FCN (xdgamma, XDGAMMA) (const double&, double&);
 
   int F77_FCN (dlgams, DLGAMS) (const double&, double&, double&);
 }
@@ -150,7 +150,7 @@ double
 xisnan (double x)
 {
 #if defined (HAVE_ISNAN)
-  return (double) isnan (x);
+  return (double) (isnan (x) != 0);
 #else
   return 0;
 #endif
@@ -160,7 +160,7 @@ double
 xfinite (double x)
 {
 #if defined (HAVE_FINITE)
-  return (double) finite (x);
+  return (double) (finite (x) != 0);
 #elif defined (HAVE_ISINF) && defined (HAVE_ISNAN)
   return (double) (! isinf (x) && ! isnan (x));
 #else
