@@ -20,12 +20,20 @@
    is generally kept in a file called COPYING or LICENSE.  If you do not
    have a copy of the license, write to the Free Software Foundation,
    675 Mass Ave, Cambridge, MA 02139, USA. */
-
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#define READLINE_LIBRARY
+
 /* **************************************************************** */
 /*								    */
 /*			VI Emulation Mode			    */
 /*								    */
 /* **************************************************************** */
+#include "rlconf.h"
+
 #if defined (VI_MODE)
 
 #include <sys/types.h>
@@ -36,11 +44,9 @@
 #  include "ansi_stdlib.h"
 #endif /* HAVE_STDLIB_H */
 
-#if defined (STATIC_MALLOC)
-static char *xmalloc (), *xrealloc ();
-#else
-extern char *xmalloc (), *xrealloc ();
-#endif /* STATIC_MALLOC */
+#if defined (HAVE_UNISTD_H)
+#  include <unistd.h>
+#endif
 
 #include <stdio.h>
 
@@ -49,12 +55,8 @@ extern char *xmalloc (), *xrealloc ();
 #include "readline.h"
 #include "history.h"
 
-#ifndef digit
-#define digit(c)  ((c) >= '0' && (c) <= '9')
-#endif
-
-#ifndef isletter
-#define isletter(c) (((c) >= 'A' && (c) <= 'Z') || ((c) >= 'a' && (c) <= 'z'))
+#ifndef digit_p
+#define digit_p(c)  ((c) >= '0' && (c) <= '9')
 #endif
 
 #ifndef digit_value
@@ -62,21 +64,27 @@ extern char *xmalloc (), *xrealloc ();
 #endif
 
 #ifndef member
-#define member(c, s) ((c) ? (char *)strchr ((s), (c)) : 0)
+#define member(c, s) ((c) ? (char *)strchr ((s), (c)) != (char *)NULL : 0)
 #endif
 
 #ifndef isident
-#define isident(c) ((isletter(c) || digit(c) || c == '_'))
+#define isident(c) ((pure_alphabetic (c) || digit_p (c) || c == '_'))
 #endif
 
 #ifndef exchange
-#define exchange(x, y) {int temp = x; x = y; y = temp;}
+#define exchange(x, y) do {int temp = x; x = y; y = temp;} while (0)
 #endif
 
 #ifndef VI_COMMENT_BEGIN_DEFAULT
 #define VI_COMMENT_BEGIN_DEFAULT "#"
 #endif
 
+#if defined (STATIC_MALLOC)
+static char *xmalloc (), *xrealloc ();
+#else
+extern char *xmalloc (), *xrealloc ();
+#endif /* STATIC_MALLOC */
+
 /* Variables imported from readline.c */
 extern int rl_point, rl_end, rl_mark, rl_done;
 extern FILE *rl_instream;
@@ -86,11 +94,13 @@ extern char *rl_prompt;
 extern char *rl_line_buffer;
 extern int rl_arg_sign;
 
+extern void _rl_dispatch ();
+
 extern void rl_extend_line_buffer ();
-extern int rl_vi_check (), rl_digit_loop1 ();
+extern int rl_vi_check ();
 
 /* Non-zero means enter insertion mode. */
-int _rl_vi_doing_insert = 0;
+static int _rl_vi_doing_insert = 0;
 
 /* String inserted into the line by rl_vi_comment (). */
 char *rl_vi_comment_begin = (char *)NULL;
@@ -106,19 +116,14 @@ static Keymap vi_replace_map = (Keymap)NULL;
 /* The number of characters inserted in the last replace operation. */
 static int vi_replace_count = 0;
 
-/* Defined in this file */
-int rl_vi_char_search ();
+/* If non-zero, we have text inserted after a c[motion] command that put
+   us implicitly into insert mode.  Some people want this text to be
+   attached to the command so that it is `redoable' with `.'. */
+static int vi_continued_command = 0;
 
-/* Defined in display.c */
-extern int rl_message (), rl_clear_message ();
-extern void rl_redisplay ();
-
-/* Defined in complete.c */
-extern int rl_complete_internal ();
-
-int _rl_vi_last_command = 'i';	/* default `.' puts you in insert mode */
-int _rl_vi_last_repeat = 1;
-int _rl_vi_last_arg_sign = 1;
+static int _rl_vi_last_command = 'i';	/* default `.' puts you in insert mode */
+static int _rl_vi_last_repeat = 1;
+static int _rl_vi_last_arg_sign = 1;
 static int _rl_vi_last_motion = 0;
 static int _rl_vi_last_search_char = 0;
 static int _rl_vi_last_replacement = 0;
@@ -128,8 +133,10 @@ static int vi_redoing = 0;
 /* Text modification commands.  These are the `redoable' commands. */
 static char *vi_textmod = "_*\\AaIiCcDdPpYyRrSsXx~";
 
+static int rl_digit_loop1 ();
+
 void
-rl_vi_set_last ()
+_rl_vi_reset_last ()
 {
   _rl_vi_last_command = 'i';
   _rl_vi_last_repeat = 1;
@@ -137,18 +144,26 @@ rl_vi_set_last ()
   _rl_vi_last_motion = 0;
 }
 
+void
+_rl_vi_set_last (key, repeat, sign)
+     int key, repeat, sign;
+{
+  _rl_vi_last_command = key;
+  _rl_vi_last_repeat = repeat;
+  _rl_vi_last_arg_sign = sign;
+}
+
 /* Is the command C a VI mode text modification command? */
 int
 rl_vi_textmod_command (c)
      int c;
 {
-  return (member (c, vi_textmod) != (char *)NULL);
+  return (member (c, vi_textmod));
 }
 
 /* Bound to `.'.  Called from command mode, so we know that we have to
    redo a text modification command.  The default for _rl_vi_last_command
    puts you back into insert mode. */
-int
 rl_vi_redo (count, c)
      int count, c;
 {
@@ -159,20 +174,16 @@ rl_vi_redo (count, c)
     }
 
   vi_redoing = 1;
-  rl_dispatch (_rl_vi_last_command, _rl_keymap);
+  _rl_dispatch (_rl_vi_last_command, _rl_keymap);
   vi_redoing = 0;
 
   return (0);
 }
     
 /* Yank the nth arg from the previous line into this line at point. */
-int
-rl_vi_yank_arg (count)
-     int count;
+rl_vi_yank_arg (count, key)
+     int count, key;
 {
-  /* vi mode is defined to insert a space before the last argument. */
-  rl_insert (1, ' ');
-
   /* Readline thinks that the first word on a line is the 0th, while vi
      thinks the first word on a line is the 1st.  Compensate. */
   if (rl_explicit_arg)
@@ -180,13 +191,11 @@ rl_vi_yank_arg (count)
   else
     rl_yank_nth_arg ('$', 0);
 
-  rl_vi_insertion_mode ();
   return (0);
 }
 
 /* With an argument, move back that many history lines, else move to the
    beginning of history. */
-int
 rl_vi_fetch_history (count, c)
      int count, c;
 {
@@ -211,7 +220,6 @@ rl_vi_fetch_history (count, c)
 }
 
 /* Search again for the last thing searched for. */
-int
 rl_vi_search_again (count, key)
      int count, key;
 {
@@ -229,7 +237,6 @@ rl_vi_search_again (count, key)
 }
 
 /* Do a vi style search. */
-int
 rl_vi_search (count, key)
      int count, key;
 {
@@ -251,7 +258,6 @@ rl_vi_search (count, key)
 }
 
 /* Completion, from vi's point of view. */
-int
 rl_vi_complete (ignore, key)
      int ignore, key;
 {
@@ -272,22 +278,24 @@ rl_vi_complete (ignore, key)
     rl_complete (0, key);
 
   if (key == '*' || key == '\\')
-    rl_vi_insertion_mode ();
+    {
+      _rl_vi_set_last (key, 1, rl_arg_sign);
+      rl_vi_insertion_mode (1, key);
+    }
   return (0);
 }
 
 /* Tilde expansion for vi mode. */
-int
 rl_vi_tilde_expand (ignore, key)
      int ignore, key;
 {
   rl_tilde_expand (0, key);
-  rl_vi_insertion_mode ();
+  _rl_vi_set_last (key, 1, rl_arg_sign);	/* XXX */
+  rl_vi_insertion_mode (1, key);
   return (0);
 }
 
 /* Previous word in vi mode. */
-int
 rl_vi_prev_word (count, key)
      int count, key;
 {
@@ -309,7 +317,6 @@ rl_vi_prev_word (count, key)
 }
 
 /* Next word in vi mode. */
-int
 rl_vi_next_word (count, key)
      int count, key;
 {
@@ -330,7 +337,6 @@ rl_vi_next_word (count, key)
 }
 
 /* Move to the end of the ?next? word. */
-int
 rl_vi_end_word (count, key)
      int count, key;
 {
@@ -348,7 +354,6 @@ rl_vi_end_word (count, key)
 }
 
 /* Move forward a word the way that 'W' does. */
-int
 rl_vi_fWord (count)
      int count;
 {
@@ -365,7 +370,6 @@ rl_vi_fWord (count)
   return (0);
 }
 
-int
 rl_vi_bWord (count)
      int count;
 {
@@ -389,7 +393,6 @@ rl_vi_bWord (count)
   return (0);
 }
 
-int
 rl_vi_eWord (count)
      int count;
 {
@@ -419,7 +422,6 @@ rl_vi_eWord (count)
   return (0);
 }
 
-int
 rl_vi_fword (count)
      int count;
 {
@@ -445,7 +447,6 @@ rl_vi_fword (count)
   return (0);
 }
 
-int
 rl_vi_bword (count)
      int count;
 {
@@ -484,7 +485,6 @@ rl_vi_bword (count)
   return (0);
 }
 
-int
 rl_vi_eword (count)
      int count;
 {
@@ -509,33 +509,32 @@ rl_vi_eword (count)
   return (0);
 }
 
-int
-rl_vi_insert_beg ()
+rl_vi_insert_beg (count, key)
+     int count, key;
 {
-  rl_beg_of_line ();
-  rl_vi_insertion_mode ();
+  rl_beg_of_line (1, key);
+  rl_vi_insertion_mode (1, key);
   return (0);
 }
 
-int
-rl_vi_append_mode ()
+rl_vi_append_mode (count, key)
+     int count, key;
 {
   if (rl_point < rl_end)
-    rl_point += 1;
-  rl_vi_insertion_mode ();
+    rl_point++;
+  rl_vi_insertion_mode (1, key);
   return (0);
 }
 
-int
-rl_vi_append_eol ()
+rl_vi_append_eol (count, key)
+     int count, key;
 {
-  rl_end_of_line ();
-  rl_vi_append_mode ();
+  rl_end_of_line (1, key);
+  rl_vi_append_mode (1, key);
   return (0);
 }
 
 /* What to do in the case of C-d. */
-int
 rl_vi_eof_maybe (count, c)
      int count, c;
 {
@@ -546,38 +545,44 @@ rl_vi_eof_maybe (count, c)
 
 /* Switching from one mode to the other really just involves
    switching keymaps. */
-int
-rl_vi_insertion_mode ()
+rl_vi_insertion_mode (count, key)
+     int count, key;
 {
   _rl_keymap = vi_insertion_keymap;
   return (0);
 }
 
-int
-vi_done_inserting ()
+void
+_rl_vi_done_inserting ()
 {
   if (_rl_vi_doing_insert)
     {
       rl_end_undo_group ();
+      /* Now, the text between rl_undo_list->next->start and
+	 rl_undo_list->next->end is what was inserted while in insert
+	 mode. */
       _rl_vi_doing_insert = 0;
+      vi_continued_command = 1;
     }
-  return (0);
+  else
+    vi_continued_command = 0;
 }
 
-int
-rl_vi_movement_mode ()
+rl_vi_movement_mode (count, key)
+     int count, key;
 {
   if (rl_point > 0)
     rl_backward (1);
 
-  rl_vi_set_last ();
+#if 0
+  _rl_vi_reset_last ();
+#endif
 
   _rl_keymap = vi_movement_keymap;
-  vi_done_inserting ();
+  _rl_vi_done_inserting ();
   return (0);
 }
 
-int
 rl_vi_arg_digit (count, c)
      int count, c;
 {
@@ -587,7 +592,6 @@ rl_vi_arg_digit (count, c)
     return (rl_digit_argument (count, c));
 }
 
-int
 rl_vi_change_case (count, ignore)
      int count, ignore;
 {
@@ -625,7 +629,6 @@ rl_vi_change_case (count, ignore)
   return (0);
 }
 
-int
 rl_vi_put (count, key)
      int count, key;
 {
@@ -637,7 +640,6 @@ rl_vi_put (count, key)
   return (0);
 }
 
-int
 rl_vi_check ()
 {
   if (rl_point && rl_point == rl_end)
@@ -645,9 +647,8 @@ rl_vi_check ()
   return (0);
 }
 
-int
-rl_vi_column (count)
-     int count;
+rl_vi_column (count, key)
+     int count, key;
 {
   if (count > rl_end)
     rl_end_of_line ();
@@ -669,7 +670,7 @@ rl_vi_domove (key, nextkey)
 
   if (!member (c, vi_motion))
     {
-      if (digit (c))
+      if (digit_p (c))
 	{
 	  save = rl_numeric_arg;
 	  rl_numeric_arg = digit_value (c);
@@ -678,9 +679,7 @@ rl_vi_domove (key, nextkey)
 	  c = rl_read_key ();	/* real command */
 	  *nextkey = c;
 	}
-      else if ((key == 'd' && c == 'd') ||
-	       (key == 'y' && c == 'y') ||
-	       (key == 'c' && c == 'c'))
+      else if (key == c && (key == 'd' || key == 'y' || key == 'c'))
 	{
 	  rl_mark = rl_end;
 	  rl_beg_of_line ();
@@ -699,7 +698,7 @@ rl_vi_domove (key, nextkey)
   rl_line_buffer[rl_end++] = ' ';
   rl_line_buffer[rl_end] = '\0';
 
-  rl_dispatch (c, _rl_keymap);
+  _rl_dispatch (c, _rl_keymap);
 
   /* Remove the blank that we added. */
   rl_end = old_end;
@@ -714,23 +713,30 @@ rl_vi_domove (key, nextkey)
   /* rl_vi_f[wW]ord () leaves the cursor on the first character of the next
      word.  If we are not at the end of the line, and we are on a
      non-whitespace character, move back one (presumably to whitespace). */
-  if ((c == 'w' || c == 'W') && (rl_point < rl_end) &&
+  if ((to_upper (c) == 'W') && rl_point < rl_end && rl_point > rl_mark &&
       !whitespace (rl_line_buffer[rl_point]))
     rl_point--;
 
   /* If cw or cW, back up to the end of a word, so the behaviour of ce
      or cE is the actual result.  Brute-force, no subtlety. */
-  if (key == 'c' && (to_upper (c) == 'W'))
+  if (key == 'c' && rl_point >= rl_mark && (to_upper (c) == 'W'))
     {
-      while (rl_point && whitespace (rl_line_buffer[rl_point]))
+      /* Don't move farther back than where we started. */
+      while (rl_point > rl_mark && whitespace (rl_line_buffer[rl_point]))
 	rl_point--;
 
-      /* Move past the end of the word so that the kill doesn't remove the
-	 last letter of the previous word.  Only do this if we are not at
-	 the end of the line. */
-      if ((rl_point >= 0) && (rl_point < (rl_end - 1)) &&
-	  !whitespace (rl_line_buffer[rl_point]))
-	rl_point++;
+      /* Posix.2 says that if cw or cW moves the cursor towards the end of
+	 the line, the character under the cursor should be deleted. */
+      if (rl_point == rl_mark)
+        rl_point++;
+      else
+	{
+	  /* Move past the end of the word so that the kill doesn't
+	     remove the last letter of the previous word.  Only do this
+	     if we are not at the end of the line. */
+	  if (rl_point >= 0 && rl_point < (rl_end - 1) && !whitespace (rl_line_buffer[rl_point]))
+	    rl_point++;
+	}
     }
 
   if (rl_mark < rl_point)
@@ -741,7 +747,7 @@ rl_vi_domove (key, nextkey)
 
 /* A simplified loop for vi. Don't dispatch key at end.
    Don't recognize minus sign? */
-int
+static int
 rl_digit_loop1 ()
 {
   int key, c;
@@ -759,7 +765,7 @@ rl_digit_loop1 ()
 	}
 
       c = UNMETA (c);
-      if (numeric (c))
+      if (digit_p (c))
 	{
 	  if (rl_explicit_arg)
 	    rl_numeric_arg = (rl_numeric_arg * 10) + digit_value (c);
@@ -777,7 +783,6 @@ rl_digit_loop1 ()
   return (0);
 }
 
-int
 rl_vi_delete_to (count, key)
      int count, key;
 {
@@ -796,14 +801,13 @@ rl_vi_delete_to (count, key)
 
   /* These are the motion commands that do not require adjusting the
      mark. */
-  if ((strchr (" l|h^0%bB", c) == 0) && (rl_mark < rl_end))
+  if ((strchr (" l|h^0bB", c) == 0) && (rl_mark < rl_end))
     rl_mark++;
 
   rl_kill_text (rl_point, rl_mark);
   return (0);
 }
 
-int
 rl_vi_change_to (count, key)
      int count, key;
 {
@@ -825,22 +829,23 @@ rl_vi_change_to (count, key)
   /* These are the motion commands that do not require adjusting the
      mark.  c[wW] are handled by special-case code in rl_vi_domove(),
      and already leave the mark at the correct location. */
-  if ((strchr (" l|hwW^0%bB", c) == 0) && (rl_mark < rl_end))
+  if ((strchr (" l|hwW^0bB", c) == 0) && (rl_mark < rl_end))
     rl_mark++;
 
   /* The cursor never moves with c[wW]. */
   if ((to_upper (c) == 'W') && rl_point < start_pos)
     rl_point = start_pos;
 
+  rl_kill_text (rl_point, rl_mark);
+
   rl_begin_undo_group ();
   _rl_vi_doing_insert = 1;
-  rl_kill_text (rl_point, rl_mark);
-  rl_vi_insertion_mode ();
+  _rl_vi_set_last (key, count, rl_arg_sign);
+  rl_vi_insertion_mode (1, key);
 
   return (0);
 }
 
-int
 rl_vi_yank_to (count, key)
      int count, key;
 {
@@ -869,9 +874,8 @@ rl_vi_yank_to (count, key)
   return (0);
 }
 
-int
-rl_vi_delete (count)
-     int count;
+rl_vi_delete (count, key)
+     int count, key;
 {
   int end;
 
@@ -895,8 +899,8 @@ rl_vi_delete (count)
 
 /* Turn the current line into a comment in shell history.
    A K*rn shell style function. */
-int
-rl_vi_comment ()
+rl_vi_comment (count, key)
+     int count, key;
 {
   rl_beg_of_line ();
 
@@ -906,11 +910,16 @@ rl_vi_comment ()
     rl_insert_text (VI_COMMENT_BEGIN_DEFAULT);	/* Default. */
 
   rl_redisplay ();
-  rl_newline (1, '\010');
+  rl_newline (1, '\n');
   return (0);
 }
 
-int
+rl_vi_first_print (count, key)
+     int count, key;
+{
+  return (rl_back_to_indent ());
+}
+
 rl_back_to_indent (ignore1, ignore2)
      int ignore1, ignore2;
 {
@@ -920,19 +929,12 @@ rl_back_to_indent (ignore1, ignore2)
   return (0);
 }
 
-int
-rl_vi_first_print ()
-{
-  return (rl_back_to_indent ());
-}
-
 /* NOTE: it is necessary that opposite directions are inverses */
 #define	FTO	 1		/* forward to */
 #define BTO	-1		/* backward to */
 #define FFIND	 2		/* forward find */
 #define BFIND	-2		/* backward find */
 
-int
 rl_vi_char_search (count, key)
      int count, key;
 {
@@ -978,7 +980,7 @@ rl_vi_char_search (count, key)
 	  if (pos == 0)
 	    {
 	      ding ();
-	      return 0;
+	      return -1;
 	    }
 
 	  pos--;
@@ -1034,8 +1036,8 @@ rl_vi_char_search (count, key)
 }
 
 /* Match brackets */
-int
-rl_vi_match ()
+rl_vi_match (ignore, key)
+     int ignore, key;
 {
   int count = 1, brack, pos;
 
@@ -1114,7 +1116,6 @@ rl_vi_bracktype (c)
     }
 }
 
-int
 rl_vi_change_char (count, key)
      int count, key;
 {
@@ -1142,12 +1143,10 @@ rl_vi_change_char (count, key)
   return (0);
 }
 
-int
 rl_vi_subst (count, key)
      int count, key;
 {
   rl_begin_undo_group ();
-  _rl_vi_doing_insert = 1;
 
   if (uppercase_p (key))
     {
@@ -1155,15 +1154,19 @@ rl_vi_subst (count, key)
       rl_kill_line (1);
     }
   else
-    rl_delete (count, key);
+    rl_delete_text (rl_point, rl_point+count);
 
   rl_end_undo_group ();
 
-  rl_vi_insertion_mode ();
+  _rl_vi_set_last (key, count, rl_arg_sign);
+
+  rl_begin_undo_group ();
+  _rl_vi_doing_insert = 1;
+  rl_vi_insertion_mode (1, key);
+
   return (0);
 }
 
-int
 rl_vi_overstrike (count, key)
      int count, key;
 {
@@ -1193,7 +1196,6 @@ rl_vi_overstrike (count, key)
   return (0);
 }
 
-int
 rl_vi_overstrike_delete (count)
      int count;
 {
@@ -1224,7 +1226,6 @@ rl_vi_overstrike_delete (count)
   return (0);
 }
 
-int
 rl_vi_replace (count, key)
      int count, key;
 {
@@ -1236,7 +1237,7 @@ rl_vi_replace (count, key)
     {
       vi_replace_map = rl_make_bare_keymap ();
 
-      for (i = ' '; i < 127; i++)
+      for (i = ' '; i < KEYMAP_SIZE; i++)
 	vi_replace_map[i].function = rl_vi_overstrike;
 
       vi_replace_map[RUBOUT].function = rl_vi_overstrike_delete;
@@ -1256,19 +1257,19 @@ rl_vi_replace (count, key)
   return (0);
 }
 
+#if 0
 /* Try to complete the word we are standing on or the word that ends with
    the previous character.  A space matches everything.  Word delimiters are
    space and ;. */
-int
-rl_vi_possible_completions ()
+rl_vi_possible_completions()
 {
   int save_pos = rl_point;
 
   if (rl_line_buffer[rl_point] != ' ' && rl_line_buffer[rl_point] != ';')
     {
-      while (rl_line_buffer[rl_point] != ' ' &&
+      while (rl_point < rl_end && rl_line_buffer[rl_point] != ' ' &&
 	     rl_line_buffer[rl_point] != ';')
-	;
+	rl_point++;
     }
   else if (rl_line_buffer[rl_point - 1] == ';')
     {
@@ -1281,6 +1282,7 @@ rl_vi_possible_completions ()
 
   return (0);
 }
+#endif
 
 #if defined (STATIC_MALLOC)
 
