@@ -1,7 +1,6 @@
-// SLList-plot.cc                                          -*- C++ -*-
 /*
 
-Copyright (C) 1993, 1994, 1995 John W. Eaton
+Copyright (C) 1996 John W. Eaton
 
 This file is part of Octave.
 
@@ -17,15 +16,20 @@ for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
 // Instantiate Lists of various values.
 
-#include <SLList.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
-#include "tree-plot.h"
+#include "SLList.h"
+#include "SLList.cc"
+
+#include "pt-plot.h"
 
 template class SLNode<subplot *>;
 template class SLList<subplot *>;
@@ -33,6 +37,5 @@ template class SLList<subplot *>;
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
-;;; page-delimiter: "^/\\*" ***
 ;;; End: ***
 */
