@@ -403,7 +403,7 @@ imagepath (void)
 }
 
 static int
-loadpath (void)
+octave_loadpath (void)
 {
   int status = 0;
 
@@ -429,7 +429,7 @@ symbols_of_defaults (void)
   DEFVAR (EXEC_PATH, Vexec_path, 0, exec_path,
     "colon separated list of directories to search for programs to run");
 
-  DEFVAR (LOADPATH, Vload_path, 0, loadpath,
+  DEFVAR (LOADPATH, Vload_path, 0, octave_loadpath,
     "colon separated list of directories to search for scripts");
 
   DEFVAR (IMAGEPATH, OCTAVE_IMAGEPATH, 0, imagepath,
