@@ -1,7 +1,6 @@
-// help.h                                              -*- C++ -*-
 /*
 
-Copyright (C) 1992, 1993, 1994, 1995 John W. Eaton
+Copyright (C) 1996 John W. Eaton
 
 This file is part of Octave.
 
@@ -17,7 +16,7 @@ for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
@@ -26,27 +25,42 @@ Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 class ostrstream;
 
-typedef struct help_list
+#include <string>
+
+class string_vector;
+
+// XXX FIXME XXX -- should probably use string, not char*.
+
+struct help_list
 {
   char *name;
   char *help;
 };
 
-extern char **names (help_list *l, int& count);
+extern string_vector names (help_list *l, int& count);
 extern help_list *operator_help (void);
 extern help_list *keyword_help (void);
 
+extern void print_usage (const string& nm, int just_usage = 0);
+
 extern int help_from_list (ostrstream& output_buf,
-			   const help_list *list, const char *string,
+			   const help_list *list, const string& nm,
 			   int usage);
 
-extern void print_usage (const char *s, int just_usage = 0);
+// Name of the info file specified on command line.
+// (--info-file file)
+extern string Vinfo_file;
+
+// Name of the info reader we'd like to use.
+// (--info-program program)
+extern string Vinfo_prog;
+
+extern void symbols_of_help (void);
 
 #endif
 
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
-;;; page-delimiter: "^/\\*" ***
 ;;; End: ***
 */
