@@ -134,13 +134,6 @@ symbol_def::is_builtin_function (void) const
   return (type & BUILTIN_FUNCTION);
 }
 
-int
-symbol_def::is_dynamically_loaded_function (void) const
-{
-  return ((type & BUILTIN_FUNCTION) &&
-	  definition->is_dynamically_loaded_function ());
-}
-
 // XXX FIXME XXX
 int
 symbol_def::is_map_element (const string& /* elts */) const
@@ -306,12 +299,6 @@ symbol_record::is_builtin_function (void) const
   return definition ? definition->is_builtin_function () : 0;
 }
 
-int
-symbol_record::is_dynamically_loaded_function (void) const
-{
-  return definition ? definition->is_dynamically_loaded_function () : 0;
-}
-
 int
 symbol_record::is_variable (void) const
 {
@@ -617,9 +604,7 @@ symbol_record::variable_value (void) const
   if (is_variable ())
     {
       tree_constant *tmp = (tree_constant *) def ();
-
-      if (tmp)
-	retval = tmp->value ();
+      retval = tmp->value ();
     }
 
   return retval;
@@ -932,7 +917,6 @@ symbol_table::rename (const string& old_name, const string& new_name)
 	  prev->chain (ptr->next ());
 
 	  index = hash (new_name) & HASH_MASK;
-	  ptr->chain (table[index].next ());
 	  table[index].chain (ptr);
 	  ptr->rename (new_name);
 
@@ -957,9 +941,7 @@ symbol_table::clear (int clear_user_functions)
       while (ptr)
 	{
 	  if (ptr->is_user_variable ()
-	      || (clear_user_functions
-		  && (ptr->is_user_function ()
-		      || ptr->is_dynamically_loaded_function ())))
+	      || (clear_user_functions && ptr->is_user_function ()))
 	    {
 	      ptr->clear ();
 	    }
@@ -980,9 +962,7 @@ symbol_table::clear (const string& nm, int clear_user_functions)
     {
       if (ptr->name () == nm
 	  && (ptr->is_user_variable ()
-	      || (clear_user_functions
-		  && (ptr->is_user_function ()
-		      || ptr->is_dynamically_loaded_function ()))))
+	      || (clear_user_functions && ptr->is_user_function ())))
 	{
 	  ptr->clear ();
 	  return 1;
@@ -1018,9 +998,7 @@ pstrcmp (char **a, char **b)
 static inline int
 symbol_record_info_cmp (symbol_record_info *a, symbol_record_info *b)
 {
-  string a_nm = a->name ();
-  string b_nm = b->name ();
-  return a_nm.compare (b_nm);
+  return (a->name () == b->name ());
 }
 
 static int
