@@ -153,34 +153,6 @@ SCO_init (void)
 }
 #endif
 
-// XXX FIXME XXX -- some systems define struct __exception.
-
-#if defined (EXCEPTION_IN_MATH)
-extern "C"
-int
-matherr (struct exception *x)
-{
-  // Possibly print our own message someday.  Should probably be
-  // user-switchable.
-
-  switch (x->type)
-    {
-    case DOMAIN:
-    case SING:
-    case OVERFLOW:
-    case UNDERFLOW:
-    case TLOSS:
-    case PLOSS:
-    default:
-      break;
-    }
-
-  // But don't print the system message.
-
-  return 1;
-}
-#endif
-
 void
 sysdep_init (void)
 {
@@ -363,16 +335,23 @@ string
 octave_getcwd (void)
 {
   string retval;
+
   char buf[MAXPATHLEN];
 
+  char *tmp = 0;
+
 #if defined (__EMX__)
-  char *tmp = _getcwd2 (buf, MAXPATHLEN);
-#else
-  char *tmp = getcwd (buf, MAXPATHLEN);
+  tmp = _getcwd2 (buf, MAXPATHLEN);
+#elif defined (HAVE_GETWD)
+  tmp = getwd (buf);
+#elif defined (HAVE_GETCWD)
+  tmp = getcwd (buf, MAXPATHLEN);
 #endif
 
   if (tmp)
     retval = tmp;
+  else
+    error ("unable to find current directory");
 
   return retval;
 }
