@@ -1582,7 +1582,9 @@ __error_text__ is guaranteed to be the empty string.");
   DEFVAR (history_file, default_history_file (), 0, history_file,
     "name of command history file");
 
-  DEFVAR (history_size, default_history_size (), 0, history_size,
+  double tmp_hist_size = default_history_size ();
+
+  DEFVAR (history_size, tmp_hist_size, 0, history_size,
     "number of commands to save in the history list");
 
   DEFVAR (ignore_function_time_stamp, "system", 0, ignore_function_time_stamp,
@@ -1619,6 +1621,7 @@ install_builtin_variables (void)
   symbols_of_pt_mat ();
   symbols_of_pt_plot ();
   symbols_of_syscalls ();
+  symbols_of_toplev ();
   symbols_of_value ();
   symbols_of_variables ();
 }
@@ -1630,7 +1633,7 @@ DEFUN_TEXT (clear, args, ,
 \n\
 Clear symbol(s) matching a list of globbing patterns.\n\
 \n\
-If no arguments are given, clear all user-defined variables and
+If no arguments are given, clear all user-defined variables and\n\
 functions.\n\
 \n\
 With -x, exclude the named variables")
